#!/usr/bin/python
import sys
import subprocess
import collections
import matplotlib
import matplotlib.pyplot as plt
from statistics import median, mean

from collections import OrderedDict

linestyles = OrderedDict(
    [('solid',               (0, ())),
     ('loosely dotted',      (0, (1, 10))),
     ('dotted',              (0, (1, 5))),
     ('densely dotted',      (0, (1, 1))),

     ('loosely dashed',      (0, (5, 10))),
     ('dashed',              (0, (5, 5))),
     ('densely dashed',      (0, (5, 1))),

     ('loosely dashdotted',  (0, (3, 10, 1, 10))),
     ('dashdotted',          (0, (3, 5, 1, 5))),
     ('densely dashdotted',  (0, (3, 1, 1, 1))),

     ('loosely dashdotdotted', (0, (3, 10, 1, 10, 1, 10))),
     ('dashdotdotted',         (0, (3, 5, 1, 5, 1, 5))),
     ('densely dashdotdotted', (0, (3, 1, 1, 1, 1, 1)))])

def calcSpeedup(timesRef, timesComp):
    speedup = []
    for i in range(min(len(timesRef),len(timesComp))):
        speedup.append(timesRef[i]/timesComp[i])
    return speedup

def chooseCompiler(gccStr, clangStr, data):
    speed = calcSpeedup(data[gccStr], data[clangStr])
    if mean(speed) > 1.0:
        return clangStr
    else:
        return gccStr

def calcAndPrintSpeedup(firstLabel, secondLabel, data):
    speedup = calcSpeedup(data[firstLabel], data[secondLabel])
    print(firstLabel+' (best) vs '+secondLabel+' (best): ')
    for i in range(len(speedup)):
        print(str(minN+i)+", "+str(speedup[i]))
    print('Average '+firstLabel+' (best) vs '+secondLabel+' (best): '+str(mean(speedup)))
    return speedup

cmds = ['original-gcc', 'original-clang', 'original-noO3-gcc', 'original-noO3-clang', 'manually-unrolled-gcc', 'manually-unrolled-clang', 'compiler-unrolled-gcc', 'compiler-unrolled-clang', 'ADD-ED16-gcc', 'ADD-ED16-clang', 'ADD-ED32-gcc', 'ADD-ED32-clang', 'ADD-ED64-gcc', 'ADD-ED64-clang', 'ADD-ED-noO3-gcc', 'ADD-ED-noO3-clang']

data = collections.defaultdict(list)

maxN = 2000
minN = 1
stepSize = 1
numExec = 50

xran = range(minN,maxN+1,stepSize)

maxVal = 0

for i in xran:
    print(i)
    results = []
    for cmd in cmds:
        times = []
        fibRes = 0
        for ai in range(numExec):
            timeRes = subprocess.run(['build/'+cmd, str(i)], stdout=subprocess.PIPE).stdout.decode('utf-8')
            resSplit = timeRes.split('\n')
            times.append(int(resSplit[0]))
            fibRes = int(resSplit[1])
        tmed = median(times)
        data[cmd].append(tmed)
        results.append(fibRes)
        maxVal = max(maxVal, tmed)
    if len(set(results)) != 1:
        print(results)
        break


usedLabels = []

usedLabels.append(chooseCompiler('original-gcc', 'original-clang', data))
usedLabels.append(chooseCompiler('manually-unrolled-gcc', 'manually-unrolled-clang', data))
usedLabels.append(chooseCompiler('compiler-unrolled-gcc', 'compiler-unrolled-clang', data))
usedLabels.append(chooseCompiler('ADD-ED16-gcc', 'ADD-ED16-clang', data))
usedLabels.append(chooseCompiler('ADD-ED32-gcc', 'ADD-ED32-clang', data))
usedLabels.append(chooseCompiler('ADD-ED64-gcc', 'ADD-ED64-clang', data))
usedLabels.append(chooseCompiler('original-noO3-gcc', 'original-noO3-clang', data))
usedLabels.append(chooseCompiler('ADD-ED-noO3-gcc', 'ADD-ED-noO3-clang', data))

for i in range(len(usedLabels)):
    for j in range(i+1, len(usedLabels)):
        calcAndPrintSpeedup(usedLabels[i], usedLabels[j], data)
        calcAndPrintSpeedup(usedLabels[j], usedLabels[i], data)

def plotAndSave(usedLabels, fileName, log=False):
    plt.clf()
    plt.figure(figsize=(10, 7))
    ymin = 0
    if log:
        plt.yscale('log')
        ymin = 10

    locMaxVal = 0
    for l in usedLabels:
        locMaxVal = max(locMaxVal, max(data[l]))

    ls = ['solid','solid','solid','solid','solid','solid','solid','solid']
    col = ['black','red','yellow','green','blue','orange','pink','purple']

    plt.ylim((ymin,locMaxVal+1))
    plt.xlim((minN,max(xran)))
    plt.xlabel('n', fontsize=18)
    plt.ylabel('Execution time of fib(n) in ns', fontsize=18)
    plt.tick_params(labelsize=18)

    for label in zip(usedLabels, ls[:len(usedLabels)], col[:len(usedLabels)]):
        plt.plot(xran, data[label[0]], linestyle=linestyles[label[1]], color=label[2])

    plt.legend(usedLabels, fontsize=18, bbox_to_anchor=(1.04,1), loc="upper left")
    plt.savefig(fileName+".pdf", bbox_inches="tight")

plotAndSave(usedLabels, "allInOne", False)
plotAndSave(usedLabels, "allInOne-log", True)

plotAndSave(usedLabels[:6], "onlyOptimized", False)
plotAndSave(usedLabels[:6], "onlyOptimized-log", True)

plotAndSave(usedLabels[6:], "onlyUnoptimized", False)
plotAndSave(usedLabels[6:], "onlyUnoptimized-log", True)